#include "ParserMp2a.h"

const uint16_t ParserMp2a::FrequencyTable[3] = { 44100, 48000, 32000 };

const uint16_t ParserMp2a::BitrateTable[2][3][15] =
{ { {0, 32, 64, 96, 128, 160, 192, 224, 256, 288, 320, 352, 384, 416, 448 },
  {0, 32, 48, 56,  64,  80,  96, 112, 128, 160, 192, 224, 256, 320, 384 },
  {0, 32, 40, 48,  56,  64,  80,  96, 112, 128, 160, 192, 224, 256, 320 } },
{ {0, 32, 48, 56,  64,  80,  96, 112, 128, 144, 160, 176, 192, 224, 256},
  {0,  8, 16, 24,  32,  40,  48,  56,  64,  80,  96, 112, 128, 144, 160},
  {0,  8, 16, 24,  32,  40,  48,  56,  64,  80,  96, 112, 128, 144, 160} } };

const uint32_t ParserMp2a::SamplesByLayer[4] = { 0, 384, 1152, 1152 };

bool ParserMp2a::DecodeHeader(uint32_t header, MpaFrameHeader& decodedHeader)
{
	int sample_rate, frame_size, mpeg25, padding;
	int sample_rate_index, bitrate_index;

	if (!IsMpaHeader(header))
	{
		return false;
	}

	if (header & (1 << 20))
	{
		decodedHeader.lsf = ((header & (1 << 19)) != 0) ? 0 : 1;
		mpeg25 = 0;
	}
	else
	{
		decodedHeader.lsf = 1;
		mpeg25 = 1;
	}

	decodedHeader.layer = 4 - ((header >> 17) & 3);
	/* extract frequency */
	sample_rate_index = (header >> 10) & 3;
	if (sample_rate_index >= ARRSIZE(FrequencyTable))
	{
		sample_rate_index = 0;
	}

	sample_rate = FrequencyTable[sample_rate_index] >> (decodedHeader.lsf + mpeg25);
	sample_rate_index += 3 * (decodedHeader.lsf + mpeg25);
	decodedHeader.sample_rate_index = sample_rate_index;
	decodedHeader.error_protection = ((header >> 16) & 1) ^ 1;
	decodedHeader.sample_rate = sample_rate;

	bitrate_index = (header >> 12) & 0xf;
	padding = (header >> 9) & 1;
	//extension = (header >> 8) & 1;
	decodedHeader.mode = (header >> 6) & 3;
	decodedHeader.mode_ext = (header >> 4) & 3;
	//copyright = (header >> 3) & 1;
	//original = (header >> 2) & 1;
	//emphasis = header & 3;

	if (decodedHeader.mode == MPA_MONO)
	{
		decodedHeader.nb_channels = 1;
	}
	else
	{
		decodedHeader.nb_channels = 2;
	}

	if (bitrate_index != 0)
	{
		frame_size = BitrateTable[decodedHeader.lsf][decodedHeader.layer - 1][bitrate_index];

		decodedHeader.bit_rate = frame_size * 1000;
		switch (decodedHeader.layer)
		{
		case 1:
		{
			frame_size = (frame_size * 12000) / sample_rate;
			frame_size = (frame_size + padding) * 4;
		}
		break;
		case 2:
		{
			frame_size = (frame_size * 144000) / sample_rate;
			frame_size += padding;
		}
		break;
		default:
		case 3:
		{
			frame_size = (frame_size * 144000) / (sample_rate << decodedHeader.lsf);
			frame_size += padding;
		}
		break;
		}

		decodedHeader.frame_size = frame_size;
	}
	else
	{
		/* if no frame size computed, signal it */
		return false;
	}

#if defined(DEBUG)
	ff_dlog(NULL, "layer%d, %d Hz, %d kbits/s, ",
		decodedHeader.layer, decodedHeader.sample_rate, decodedHeader.bit_rate);
	if (decodedHeader.nb_channels == 2) {
		if (decodedHeader.layer == 3) {
			if (decodedHeader.mode_ext & MODE_EXT_MS_STEREO)
				ff_dlog(NULL, "ms-");
			if (decodedHeader.mode_ext & MODE_EXT_I_STEREO)
				ff_dlog(NULL, "i-");
		}
		ff_dlog(NULL, "stereo");
	}
	else {
		ff_dlog(NULL, "mono");
	}
	ff_dlog(NULL, "\n");
#endif
	return true;
}

bool ParserMp2a::VerifyBuffer(uint8_t const*  buffer, const unsigned int size)
{
	size_t processed(0);
	MpaFrameHeader decodedHeader;

	while (DecodeHeader(ParserMp2a::Rb32(buffer), decodedHeader))
	{
		buffer += decodedHeader.frame_size;
		processed += decodedHeader.frame_size;
		long long leftoverSize = size - processed;

		if (leftoverSize < 0)
		{
			break;
		}
	}

	return processed == size;
}

bool ParserMp2a::EnsureBuffer(const uint8_t*& buffer, int& size)
{	
	const int headerSize = static_cast<int>(sizeof(uint32_t));
	
	while (size > headerSize && !IsMpaHeader(ParserMp2a::Rb32(buffer)))
	{
		++buffer;
		--size;
	}

	if (size < headerSize)
	{
		return false;
	}
	
	const uint8_t* bufferIteraror = buffer;
	size_t processed(0);
	MpaFrameHeader decodedHeader;

	while (DecodeHeader(ParserMp2a::Rb32(bufferIteraror), decodedHeader))
	{
		bufferIteraror += decodedHeader.frame_size;
		processed += decodedHeader.frame_size;
		long long leftoverSize = size - processed;

		if (leftoverSize <= 0)
		{
			break;
		}
	}
	
	size = static_cast<int>(processed);

	return processed > 0;
}
