#pragma once

#define WIN32_LEAN_AND_MEAN             // Exclude rarely-used stuff from Windows headers

#include <cstdint>

#define MPA_FRAME_HEADER \
    int frame_size; \
    int error_protection; \
    int layer; \
    int sample_rate; \
    int sample_rate_index; /* between 0 and 8 */ \
    int bit_rate; \
    int nb_channels; \
    int mode; \
    int mode_ext; \
    int lsf;

typedef struct MpaFrameHeader {
	MPA_FRAME_HEADER
} MpaFrameHeader;

#ifndef ARRSIZE
#define ARRSIZE(a) ((sizeof(a))/(sizeof(*a)))
#endif

class LIVE555_API ParserMp2a
{
public:
	static inline uint32_t Rb32(const uint8_t* buffer)
	{
		return (((uint32_t)((const uint8_t*)(buffer))[0] << 24)
			| (((const uint8_t*)(buffer))[1] << 16)
			| (((const uint8_t*)(buffer))[2] << 8)
			| ((const uint8_t*)(buffer))[3]);
	}

	static inline bool IsMpaHeader(uint32_t header)
	{
		/* header */
		if ((header & 0xffe00000) != 0xffe00000)
		{
			return false;
		}

		/* version check */
		if ((header & (3 << 19)) == 1 << 19)
		{
			return false;
		}

		/* layer check */
		if ((header & (3 << 17)) == 0)
		{
			return false;
		}

		/* bit rate */
		if ((header & (0xf << 12)) == 0xf << 12)
		{
			return false;
		}

		/* frequency */
		if ((header & (3 << 10)) == 3 << 10)
		{
			return false;
		}

		return true;
	}

	static inline bool DecodeHeader(uint8_t const* buffer, MpaFrameHeader& decodedHeader)
	{
		memset(&decodedHeader, 0, sizeof(decodedHeader));
		return buffer ? DecodeHeader(Rb32(buffer), decodedHeader) : false;
	}

	static bool VerifyBuffer(uint8_t const* buffer, const unsigned int size);
	static bool EnsureBuffer(const uint8_t*& buffer, int& size);

	static inline long getFramePlayTime(const MpaFrameHeader& decodedHeader)
	{
		const uint32_t numSamples = SamplesByLayer[decodedHeader.layer];
		const uint32_t frequency = decodedHeader.sample_rate;

		// result is numSamples / frequency rounds to nearest integer
		return frequency ? static_cast<long>(((numSamples * 2 * MILLION) / frequency + 1) / 2) : 0;
	}

private:
	static bool DecodeHeader(uint32_t header, MpaFrameHeader& decodedHeader);

private:
	enum Consts { MILLION = 1000000 };
	enum Mpa { MPA_STEREO = 0, MPA_JSTEREO = 1, MPA_DUAL = 2, MPA_MONO = 3 };

	static const uint16_t FrequencyTable[3];
	static const uint16_t BitrateTable[2][3][15];
	static const uint32_t SamplesByLayer[4];
};
